import Validators from './validators'
import { regFormat } from './helpers'

const REG_URL = /^(?:(?:(?:https?):)?\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})).?)(?::\d{2,5})?(?:[/?#]\S*)?$/i

// Uses "format" internally which is already memoized
let url = regFormat(function (options) {
  let protocols = [].concat(options.protocols || options.protocol || Validators.defaultOptions.urlProtocols).join('|')
  return protocols !== 'http|https' ? new RegExp(REG_URL.source.replace('https?', protocols), REG_URL.flags) : REG_URL
}, 'url')

url.REG_URL = REG_URL

export default url

const PRIVATE_AND_LOCAL =
  '(?!(?:10|127)(?:\\.\\d{1,3}){3})' +
  '(?!(?:169\\.254|192\\.168)(?:\\.\\d{1,3}){2})' +
  '(?!172\\.(?:1[6-9]|2\\d|3[0-1])(?:\\.\\d{1,3}){2})'

function buildReg (options) {
  let protocols = [].concat(options.protocols).join('|')
  return (
    '^' +
      // protocol identifier (optional)
      // short syntax // still required
      (options.protocolIdentifier ? `(?:(?:(?:${protocols}):)?\\/\\/)` : '') +
      // user:pass BasicAuth (optional)
      (options.basicAuth ? '(?:\\S+(?::\\S*)?@)?' : '') +
      '(?:' +
      // IP address exclusion
      // private & local networks
      PRIVATE_AND_LOCAL +
      // IP address dotted notation octets
      // excludes loopback network 0.0.0.0
      // excludes reserved space >= 224.0.0.0
      // excludes network & broacast addresses
      // (first & last IP address of each class)
      '(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])' +
      '(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}' +
      '(?:\\.(?:[1-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))' +
      '|' +
      // host & domain names, may end with dot
      // can be replaced by a shortest alternative
      // (?![-_])(?:[-\\w\\u00a1-\\uffff]{0,63}[^-_]\\.)+
      '(?:' +
      '(?:' +
      '[a-z0-9\\u00a1-\\uffff]' +
      '[a-z0-9\\u00a1-\\uffff_-]{0,62}' +
      ')?' +
      '[a-z0-9\\u00a1-\\uffff]\\.' +
      ')+' +
      // TLD identifier name, may end with dot
      '(?:[a-z\\u00a1-\\uffff]{2,}\\.?)' +
      ')' +
      // port number (optional)
      '(?::\\d{2,5})?' +
      // resource path (optional)
      '(?:[/?#]\\S*)?' +
      '$',
    'i'
  )
}

// protocol identifier (optional)
const PROTOCOL = '(?:(?:https?:)?//)?'

// user:pass BasicAuth (optional)
const AUTH = '(?:S+(?::S*)?@)?'

const HOST = [
  // host & domain names, may end with dot
  // can be replaced by a shortest alternative
  // (?![-_])(?:[-\w\u00a1-\uffff]{0,63}[^-_]\.)+
  '(?:(?:[a-z0-9\u00a1-\uffff][a-z0-9\u00a1-\uffff_-]{0,62})?[a-z0-9\u00a1-\uffff].)+',

  // TLD identifier name, may end with dot
  '(?:[a-z\u00a1-\uffff]{2,}.?)'
].join('')

// IP address dotted notation octets
// excludes loopback network 0.0.0.0
// excludes reserved space >= 224.0.0.0
// excludes network & broacast addresses
// (first & last IP address of each class)
const IPV4 =
  '(?:' +
  [
    // first octet
    '(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])',

    // second and third octets
    '(?:.(?:1?d{1,2}|2[0-4]d|25[0-5])){2}',

    // fourth octet
    '(?:.(?:1dd|2[0-4]d|25[0-4]|[1-9]d?))'
  ].join('') +
  ')'

// the IPv6 matching part is from here: https://gist.github.com/syzdek/6086792
// IPv6 address in URLs are wrapped with []s
const IPV6 =
  '[(?:' +
  [
    // 1:2:3:4:5:6:7:8
    '(?:[da-f]{1,4}:){7,7}[da-f]{1,4}',

    // 2001:db8:3:4::192.0.2.33  64:ff9b::192.0.2.33  (IPv4-Embedded IPv6 Address)
    '(?:[da-f]{1,4}:){1,4}:' + IPV4,

    // ::255.255.255.255  ::ffff:255.255.255.255  ::ffff:0:255.255.255.255  (IPv4-mapped IPv6 addresses)
    '::(?:ffff(?::0{1,4}){0,1}:){0,1}' + IPV4,

    // 1::3:4:5:6:7:8  1::3:4:5:6:7:8 1::8
    '[da-f]{1,4}:(?:(?::[da-f]{1,4}){1,6})',

    // 1::4:5:6:7:8  1:2::4:5:6:7:8  1:2::8
    '(?:[da-f]{1,4}:){1,2}(?::[da-f]{1,4}){1,5}',

    // 1::5:6:7:8  1:2:3::5:6:7:8  1:2:3::8
    '(?:[da-f]{1,4}:){1,3}(?::[da-f]{1,4}){1,4}',

    // 1::6:7:8  1:2:3:4::6:7:8  1:2:3:4::8
    '(?:[da-f]{1,4}:){1,4}(?::[da-f]{1,4}){1,3}',

    // 1::7:8  1:2:3:4:5::7:8  1:2:3:4:5::8
    '(?:[da-f]{1,4}:){1,5}(?::[da-f]{1,4}){1,2}',

    // 1::8  1:2:3:4:5:6::8  1:2:3:4:5:6::8
    '(?:[da-f]{1,4}:){1,6}:[da-f]{1,4}',

    // 1::
    '(?:[da-f]{1,4}:){1,7}:',

    // ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8  ::8  ::
    ':(?:(?::[da-f]{1,4}){1,7}|:)'
  ].join('|') +
  ')]'

// port number (optional)
const PORT = '(?::d{2,5})?'

// resource path (optional)
const PATH = '(?:[/?#]S*)?'

const URL_REGEX = new RegExp(`${PROTOCOL}${AUTH}(?:${IPV4}|${IPV6}|${HOST})${PORT}${PATH}`, 'i')
