import Validators from './validators'
import { regFormat } from './helpers'

// const REG_URL = /^(?:(?:(?:https?):)?\/\/)(?:\S+(?::\S*)?@)?(?:(?!(?:10|127)(?:\.\d{1,3}){3})(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)(?:\.(?:[a-z\u00a1-\uffff0-9]-*)*[a-z\u00a1-\uffff0-9]+)*(?:\.(?:[a-z\u00a1-\uffff]{2,})).?)(?::\d{2,5})?(?:[/?#]\S*)?$/i

// user:pass BasicAuth (optional)
const BASIC_AUTH = '(?:\\S+(?::\\S*)?@)?'

const IPV4 =
  '(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]?|0)\\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]?|0)'

// IP address exclusion
// private & local networks
const PRIVATE_AND_LOCAL =
  '(?!(?:10|127)(?:\\.\\d{1,3}){3})' +
  '(?!(?:169\\.254|192\\.168)(?:\\.\\d{1,3}){2})' +
  '(?!172\\.(?:1[6-9]|2\\d|3[0-1])(?:\\.\\d{1,3}){2})'

// IP address dotted notation octets
// excludes loopback network 0.0.0.0
// excludes reserved space >= 224.0.0.0
// excludes network & broacast addresses
// (first & last IP address of each class)
const IPV4 =
  '(?:' +
  // first octet
  '(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])' +
  // second and third octets
  '(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}' +
  // fourth octet
  '(?:\\.(?:1\\d\\d|2[0-4]\\d|25[0-4]|[1-9]\\d?))' +
  ')'

// the IPv6 matching part is from here: https://gist.github.com/syzdek/6086792
// IPv6 address in URLs are wrapped with []s
const IPV6 =
  `\\[(?:` +
  [
    // 1:2:3:4:5:6:7:8
    '(?:[\\da-f]{1,4}:){7,7}[\\da-f]{1,4}',
    // 2001:db8:3:4::192.0.2.33  64:ff9b::192.0.2.33  (IPv4-Embedded IPv6 Address)
    '(?:[\\da-f]{1,4}:){1,4}:' + IPV4,
    // ::255.255.255.255  ::ffff:255.255.255.255  ::ffff:0:255.255.255.255  (IPv4-mapped IPv6 addresses)
    '::(?:ffff(?::0{1,4}){0,1}:){0,1}' + IPV4,
    // 1::3:4:5:6:7:8  1::3:4:5:6:7:8 1::8
    '[\\da-f]{1,4}:(?:(?::[\\da-f]{1,4}){1,6})',
    // 1::4:5:6:7:8  1:2::4:5:6:7:8  1:2::8
    '(?:[\\da-f]{1,4}:){1,2}(?::[\\da-f]{1,4}){1,5}',
    // 1::5:6:7:8  1:2:3::5:6:7:8  1:2:3::8
    '(?:[\\da-f]{1,4}:){1,3}(?::[\\da-f]{1,4}){1,4}',
    // 1::6:7:8  1:2:3:4::6:7:8  1:2:3:4::8
    '(?:[\\da-f]{1,4}:){1,4}(?::[\\da-f]{1,4}){1,3}',
    // 1::7:8  1:2:3:4:5::7:8  1:2:3:4:5::8
    '(?:[\\da-f]{1,4}:){1,5}(?::[\\da-f]{1,4}){1,2}',
    // 1::8  1:2:3:4:5:6::8  1:2:3:4:5:6::8
    '(?:[\\da-f]{1,4}:){1,6}:[\\da-f]{1,4}',
    // 1::
    '(?:[\\da-f]{1,4}:){1,7}:',
    // ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8  ::8  ::
    ':(?:(?::[\\da-f]{1,4}){1,7}|:)'
  ].join('|') +
  ')\\]'

// host & domain names, may end with dot
// can be replaced by a shortest alternative
// (?![-_])(?:[-\w\u00a1-\uffff]{0,63}[^-_]\.)+
const HOST =
  '(?:(?:[a-z0-9\\u00a1-\\uffff][a-z0-9\\u00a1-\\uffff_-]{0,62})?[a-z0-9\\u00a1-\\uffff]\\.)+' +
  // TLD identifier name, may end with dot
  '(?:[a-z\\u00a1-\\uffff]{2,}\\.?)'

// port number (optional)
const PORT = '(?::\\d{2,5})?'

// resource path (optional)
const PATH = '(?:[/][^\\s?#]*)?'

const QUERY_STRING = '(?:[?][^\\s#]*)?'

const HASH = '(?:[#]\\S*)?'

// Uses "format" internally which is already memoized
let url = regFormat(buildReg, 'url')

url.REG_URL = buildReg({})

export default url

const DEFAULT_OPTIONS = {
  protocols: ['http', 'https'], // TODO: not global option anymore
  protocolIdentifier: true,
  basicAuth: true,
  ipv4: true,
  ipv6: false,
  host: true,
  port: true,
  path: true
}

function parseURL(url, options) {
  let parts = url.match(buildReg(options))
  let auth = parts[2]
  let [username, password] = (auth || '').slice(0, -1).split('::')
  let port = parts[6]
  let ipv6 = parts[4]
  return {
    protocol: parts[1].replace(/:?\/\/$/, ''),
    basicAuth: auth ? { username, password } : auth,
    ipv4: parts[3],
    ipv6: ipv6 ? ipv6.slice(1, -1) : ipv6,
    host: parts[5],
    port: port ? +port.slice(1) : port,
    path: parts[7],
    query: parts[8],
    hash: parts[9]
  }
}

function group(option, regPart) {
  return `(${option ? regPart : ''})`
}

function buildReg(options) {
  options = Object.assign({}, DEFAULT_OPTIONS, options)
  let protocols = [].concat(options.protocol || options.protocols).join('|')
  return new RegExp(
    '^' +
      group(options.protocolIdentifier, `(?:(?:(?:${protocols}):)?\\/\\/)`) +
      group(options.basicAuth, BASIC_AUTH) +
      `(?:${[group(options.ipv4, IPV4), group(options.ipv6, IPV6), group(options.host, HOST)].join('|')})` +
      group(options.port, PORT) +
      group(options.path, PATH) +
      group(options.path, QUERY_STRING) +
      group(options.path, HASH) +
      '$',
    'i'
  )
}

// function buildReg (options) {
//   options = Object.assign({}, DEFAULT_OPTIONS, options)
//   let protocols = [].concat(options.protocol || options.protocols).join('|')
//   return new RegExp(
//     '^' +
//     // protocol identifier (optional)
//     // short syntax // still required
//     (options.protocolIdentifier ? `(?:(?:(?:${protocols}):)?\\/\\/)` : '') +
//     (options.basicAuth ? BASIC_AUTH : '') +
//     '(?:' +
//     (options.privateAndLocal ? '' : PRIVATE_AND_LOCAL) + // TODO: with ipv4?
//       [options.ipv4 ? IPV4 : '', options.ipv6 ? IPV6 : '', options.host ? HOST : ''].join('|') +
//       ')' +
//       (options.port ? PORT : '') +
//       (options.path ? PATH : '') +
//       '$',
//     'i'
//   )
// }
