import Validators from './validators'
import { regFormat } from './helpers'

// user:pass BasicAuth (optional)
const BASIC_AUTH = '(?:\\S+(?::\\S*)?@)?'

// IP address dotted notation octets
const IPV4 =
  '(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]?|0)\\.){3}(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]?|0)'

// the IPv6 matching part is from here: https://gist.github.com/syzdek/6086792
// IPv6 address in URLs are wrapped with []s
const IPV6 =
  `\\[(?:` +
  [
    // 1:2:3:4:5:6:7:8
    '(?:[\\da-f]{1,4}:){7,7}[\\da-f]{1,4}',
    // 2001:db8:3:4::192.0.2.33  64:ff9b::192.0.2.33  (IPv4-Embedded IPv6 Address)
    '(?:[\\da-f]{1,4}:){1,4}:' + IPV4,
    // ::255.255.255.255  ::ffff:255.255.255.255  ::ffff:0:255.255.255.255  (IPv4-mapped IPv6 addresses)
    '::(?:ffff(?::0{1,4}){0,1}:){0,1}' + IPV4,
    // 1::3:4:5:6:7:8  1::3:4:5:6:7:8 1::8
    '[\\da-f]{1,4}:(?:(?::[\\da-f]{1,4}){1,6})',
    // 1::4:5:6:7:8  1:2::4:5:6:7:8  1:2::8
    '(?:[\\da-f]{1,4}:){1,2}(?::[\\da-f]{1,4}){1,5}',
    // 1::5:6:7:8  1:2:3::5:6:7:8  1:2:3::8
    '(?:[\\da-f]{1,4}:){1,3}(?::[\\da-f]{1,4}){1,4}',
    // 1::6:7:8  1:2:3:4::6:7:8  1:2:3:4::8
    '(?:[\\da-f]{1,4}:){1,4}(?::[\\da-f]{1,4}){1,3}',
    // 1::7:8  1:2:3:4:5::7:8  1:2:3:4:5::8
    '(?:[\\da-f]{1,4}:){1,5}(?::[\\da-f]{1,4}){1,2}',
    // 1::8  1:2:3:4:5:6::8  1:2:3:4:5:6::8
    '(?:[\\da-f]{1,4}:){1,6}:[\\da-f]{1,4}',
    // 1::
    '(?:[\\da-f]{1,4}:){1,7}:',
    // ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8  ::8  ::
    ':(?:(?::[\\da-f]{1,4}){1,7}|:)'
  ].join('|') +
  ')\\]'

// host & domain names, may end with dot
// can be replaced by a shortest alternative
// (?![-_])(?:[-\w\u00a1-\uffff]{0,63}[^-_]\.)+
const HOST =
  '(?:(?:[a-z0-9\\u00a1-\\uffff][a-z0-9\\u00a1-\\uffff_-]{0,62})?[a-z0-9\\u00a1-\\uffff]\\.)+' +
  // TLD identifier name, may end with dot
  '(?:[a-z\\u00a1-\\uffff]{2,}\\.?)'

// port number (optional)
const PORT = '(?::\\d{2,5})?'

// resource path (optional)
const PATH = '(?:[/][^\\s?#]*)?'

const SEARCH = '(?:[?][^\\s#]*)?'

const HASH = '(?:[#]\\S*)?'

const DEFAULT_OPTIONS = {
  protocolIdentifier: false,
  basicAuth: false,
  local: false,
  ipv4: false,
  ipv6: false,
  host: true,
  port: false,
  path: true,
  search: true,
  hash: true
}

function parseURL (url, options) {
  let parts = url.match(buildReg(options))
  if (!parts) return null
  let auth = parts[2]
  let [username, ...password] = (auth || '').slice(0, -1).split(':')
  let ipv6 = parts[4]
  let port = parts[7]
  return {
    protocol: parts[1].replace(/:?\/\/$/, ''),
    basicAuth: auth ? { username, password: password.length ? password.join(':') : void 0 } : auth,
    ipv4: parts[3],
    ipv6: ipv6 ? ipv6.slice(1, -1) : ipv6,
    host: parts[5] || parts[6],
    port: port ? +port.slice(1) : port,
    path: parts[8],
    query: parts[9],
    hash: parts[10]
  }
}

// Uses "format" internally which is already memoized
let url = regFormat(options => {}, 'url')

url.parseURL = parseURL

export default url

function defaultOptions (options) {
  options = Object.assign(
    {
      protocols: Validators.defaultOptions.urlProtocols
    },
    DEFAULT_OPTIONS,
    options
  )
  options.protocols = [].concat(options.protocol || options.protocols).join('|')
  return options
}

function group (option, regPart) {
  return `(${option ? regPart : ''})`
}

function buildReg (options) {
  options = Object.assign({}, DEFAULT_OPTIONS, options)
  let protocols = [].concat(options.protocol || options.protocols || Validators.defaultOptions.urlProtocols).join('|')
  return new RegExp(
    '^' +
      `((?:(?:(?:${protocols}):)?\\/\\/)${options.protocolIdentifier ? '' : '?'})` +
      group(options.basicAuth, BASIC_AUTH) +
      `(?:${[
        group(options.ipv4, IPV4),
        group(options.ipv6, IPV6),
        group(options.host, HOST),
        group(options.local, 'localhost')
      ].join('|')})` +
      group(options.port, PORT) +
      group(options.path, PATH) +
      group(options.search, SEARCH) +
      group(options.hash, HASH) +
      '$',
    'i'
  )
}
